{%MainUnit ../suggestedit.pas

*****************************************************************************
                          suggesteditlistboxform.inc
                          --------------------------
                            SuggestEdit Component

                       Initial Revision  : 08.12.2015

*****************************************************************************
 This file is part of the SuggestEdit Component.
 Copyright (c) 2015 by Michael W. Vogel

 See the file COPYING.modifiedLGPL.txt, included in this distribution,
 for details about the license.
*****************************************************************************
}

procedure TSuggestEditListboxForm.ClickItem(Sender: TObject);
begin
  SelectItem;
end;

procedure TSuggestEditListboxForm.FillItemsDummy;
var
  i: Integer;
begin
  FItemsDummy.Clear;
  if Length(FCurrentEditText) = 0 then begin
    if FSuggestEdit.MaxBoxPreviewCount = -1 then
      FItemsDummy.Assign(FSuggestEdit.Items)
    else
      for i := 0 to min(FSuggestEdit.MaxBoxPreviewCount, FSuggestEdit.Items.Count) -1 do
        FItemsDummy.Add(FSuggestEdit.Items[i]);
    Exit;
  end;

  for i := 0 to Pred(FSuggestEdit.Items.Count) do
  begin
    if coCaseSensitive in FSuggestEdit.CompareOptions then
      if coAnywhere in FSuggestEdit.CompareOptions then
      begin
        if Pos(FCurrentEditText, FSuggestEdit.Items[i]) > 0 then
          FItemsDummy.Add(FSuggestEdit.Items[i]);
      end
      else
      begin
        if Pos(FCurrentEditText, FSuggestEdit.Items[i]) = 1 then
          FItemsDummy.Add(FSuggestEdit.Items[i]);
      end
    else
      if coAnywhere in FSuggestEdit.CompareOptions then
      begin
        if Pos(LowerCase(FCurrentEditText), LowerCase(FSuggestEdit.Items[i])) > 0 then
          FItemsDummy.Add(FSuggestEdit.Items[i]);
      end
      else
      begin
        if Pos(LowerCase(FCurrentEditText), LowerCase(FSuggestEdit.Items[i])) = 1 then
          FItemsDummy.Add(FSuggestEdit.Items[i]);
      end;
    if FSuggestEdit.MaxBoxPreviewCount = -1 then Continue;
    if FItemsDummy.Count = FSuggestEdit.MaxBoxPreviewCount then Break;
  end;
end;

procedure TSuggestEditListboxForm.FocusEdit(Sender: TObject);
begin
  Application.QueueAsyncCall(FocusOwner, 0);
end;

function TSuggestEditListboxForm.GetItemSelected: Boolean;
begin
  Result := FListBox.ItemIndex >= 0;
end;

function TSuggestEditListboxForm.GetSelectedItem: String;
begin
  if FListBox.ItemIndex >= 0 then
    Result := FListBox.Items[FListBox.ItemIndex]
  else
    Result := '';
end;

procedure TSuggestEditListboxForm.InitializeParentOK;
begin
  FParentPoint := FSuggestEdit.ClientToScreen(Point(0,0));
end;

procedure TSuggestEditListboxForm.MouseMoveItems(
  Sender: TObject; Shift: TShiftState; X, Y: Integer);
begin
  FListBox.ItemIndex := FListBox.ItemAtPos(Point(X, Y), True);
end;

procedure TSuggestEditListboxForm.CheckParentOK(
  Sender: TObject);
var
  aPoint: TPoint;
begin
  aPoint := FSuggestEdit.ClientToScreen(Point(0,0));
  if (aPoint.x <> FParentPoint.x) or (aPoint.y <> FParentPoint.y) then
    Visible := False;
end;

procedure TSuggestEditListboxForm.PrepareFirstShow;
var
  aPoint: TPoint;
begin
  FListBox.Items.Add(' ');
  aPoint := FSuggestEdit.ControlToScreen(Point(0, 0));
  SetBounds(aPoint.X, aPoint.Y + FSuggestEdit.Height, FSuggestEdit.Width, 10);
  FFirstShow := False;
end;

procedure TSuggestEditListboxForm.SelectItem;
begin
  if Assigned(FSuggestEdit) and (ssUpdatePreviewFirstItem in FSuggestEdit.State) then Exit;
  if Assigned(FOnSelectItem) then
    FOnSelectItem(FListBox.GetSelectedText);
end;

procedure TSuggestEditListboxForm.SetColor(Value: TColor);
begin
  inherited SetColor(Value);
  if Assigned(FListBox) then
    FListBox.Color := Value;
end;

procedure TSuggestEditListboxForm.SetVisible(Value: Boolean);
begin
  inherited SetVisible(Value);
  if not Assigned(FParentOKTimer) then Exit;
  if Value then begin
    InitializeParentOK;
    FParentOKTimer.Enabled := True;
  end
  else
    FParentOKTimer.Enabled := False;
end;

constructor TSuggestEditListboxForm.Create(AOwner: TComponent);
begin
  CreateNew(AOwner);
end;

constructor TSuggestEditListboxForm.CreateNew(AOwner: TComponent;
  Num: Integer);
begin
  inherited CreateNew(AOwner, Num);
  FSuggestEdit := TVirtualSuggestEdit(Owner);
  BorderStyle := bsNone;
  FormStyle := fsStayOnTop;
  FListBox := TListBox.Create(Self);
  FListBox.Align := alClient;
  FListBox.Parent := Self;
  FListBox.OnSelectionChange := SelectionChanged;
  FListBox.OnClick := ClickItem;
  FListBox.OnMouseMove := MouseMoveItems;
  FListBox.BorderStyle := bsSingle;
  FItemHeight := -1;
  FItemShowingCount := 0;
  FBorderHeight := -1;
  FItemsDummy := TStringList.Create;
  FItemsDummy.Sorted := False;
  FItemsDummy.Duplicates := dupIgnore;
  FCurrentEditText := '';
  FFirstShow := True;
  FParentOKTimer := TTimer.Create(Self);
  FParentOKTimer.OnTimer := CheckParentOK;
  FParentOKTimer.Interval := 100;
  FParentOKTimer.Enabled := False;
  OnActivate := FocusEdit;
end;

destructor TSuggestEditListboxForm.Destroy;
begin
  FItemsDummy.Free;
  inherited Destroy;
end;

procedure TSuggestEditListboxForm.DeleteIndex;
var
  aIndex: Integer;
begin
  aIndex := FListBox.ItemIndex;
  FItemsDummy.Delete(aIndex);
  if FItemsDummy.Count = 0 then begin
    Visible := False;
    Exit;
  end;
  if aIndex > 0 then aIndex := Pred(aIndex);
  FillBox(FCurrentEditText);
  FListBox.ItemIndex := aIndex;
end;

procedure TSuggestEditListboxForm.FillBox(const s: String);
begin
  FCurrentEditText := s;
  FillItemsDummy;
  if FItemsDummy.Count = 0 then
  begin
    Visible := False;
    if Assigned(FOnPreviewFirstItem) and Assigned(FSuggestEdit) and FSuggestEdit.SelectFirstItem then
    begin
      FSuggestEdit.State := FSuggestEdit.State + [ssUpdatePreviewFirstItem];
      FOnPreviewFirstItem(s);
      FSuggestEdit.State := FSuggestEdit.State - [ssUpdatePreviewFirstItem];
    end;
  end else begin
    if not Showing then
    begin
      if FFirstShow then PrepareFirstShow;
      FListBox.ItemIndex := -1;
      Visible := True;
    end;
    ResizeForm;
    FListBox.Items.Assign(FItemsDummy);
    if Assigned(FOnPreviewFirstItem) and Assigned(FSuggestEdit) and FSuggestEdit.SelectFirstItem then
    begin
      FSuggestEdit.State := FSuggestEdit.State + [ssUpdatePreviewFirstItem];
      FListBox.ItemIndex := 0;
      FOnPreviewFirstItem(FListBox.Items[0]);
      FSuggestEdit.State := FSuggestEdit.State - [ssUpdatePreviewFirstItem];
    end;
  end;
end;

procedure TSuggestEditListboxForm.FocusOwner(Data: PtrInt);
begin
  if Assigned(FSuggestEdit) then FSuggestEdit.DoFocus;
end;

function TSuggestEditListboxForm.GetSelectedText: String;
begin
  Result := FListBox.GetSelectedText;
end;

procedure TSuggestEditListboxForm.MoveItemIndexDown;
begin
  if FListBox.ItemIndex < FListBox.Items.Count - 1 then
    FListBox.ItemIndex := FListBox.ItemIndex + 1;
end;

procedure TSuggestEditListboxForm.MoveItemIndexPageDown;
var
  aIndex: Integer;
begin
  aIndex := FListBox.ItemIndex + FItemShowingCount;
  if aIndex > FListBox.Items.Count - 1 then aIndex := FListBox.Items.Count - 1;
  FListBox.ItemIndex := aIndex;
end;

procedure TSuggestEditListboxForm.MoveItemIndexPageUp;
var
  aIndex: Integer;
begin
  if FListBox.ItemIndex <= 0 then begin
    FListBox.ItemIndex := -1;
    Exit;
  end;
  aIndex := FListBox.ItemIndex - FItemShowingCount;
  if aIndex < 0 then aIndex := 0;
  FListBox.ItemIndex := aIndex;
end;

procedure TSuggestEditListboxForm.MoveItemIndexUp;
begin
  if FListBox.ItemIndex > -1 then
    FListBox.ItemIndex := FListBox.ItemIndex - 1;
end;

procedure TSuggestEditListboxForm.SelectionChanged(Sender: TObject; User: boolean);
begin
  if Assigned(FOnSelectionChange) then
    FOnSelectionChange(SelectedItem);
end;

procedure TSuggestEditListboxForm.ResizeForm;
var
  aHeight: Integer;
  aWidth: Integer;
  aPoint: TPoint;
begin
  if FItemHeight <= 0 then FItemHeight := FListBox.ItemHeight;
  if FItemHeight <= 0 then Exit;
  if FBorderHeight = -1 then FBorderHeight := GetSystemMetrics(SM_CYFRAME);
  if FSuggestEdit.MaxBoxPreviewCount > 0 then
    FItemShowingCount := MinIntValue([FItemsDummy.Count, (FSuggestEdit.MaxBoxHeight - FBorderHeight) div FItemHeight, FSuggestEdit.MaxBoxPreviewCount])
  else
    FItemShowingCount := MinIntValue([FItemsDummy.Count, (FSuggestEdit.MaxBoxHeight - FBorderHeight) div FItemHeight]);
  FItemShowingCount := Max(0, FItemShowingCount);
  aHeight := FItemShowingCount * FItemHeight + FBorderHeight;
  aPoint := FSuggestEdit.ControlToScreen(Point(0, 0));
  aPoint.y := aPoint.y + FSuggestEdit.Height;
  aWidth := FSuggestEdit.Width;
  if (aPoint.y + aHeight > Screen.Height) and (aPoint.y - aHeight - FSuggestEdit.Height > 0) then
    aPoint.y := aPoint.y - aHeight - FSuggestEdit.Height;
  SetBounds(aPoint.x, aPoint.y, aWidth, aHeight);
end;



