{%MainUnit ../suggestedit.pas

*****************************************************************************
                             suggestsstrings.inc
                             -------------------
                            SuggestEdit Component

                       Initial Revision  : 08.12.2015

*****************************************************************************
 This file is part of the SuggestEdit Component.
 Copyright (c) 2015 by Michael W. Vogel

 See the file COPYING.modifiedLGPL.txt, included in this distribution,
 for details about the license.
*****************************************************************************
}

function DateTimeToLongWord(aDateTime: TDateTime): DWord;
begin
  Result := Trunc(aDateTime * (1 shl 14));
end;

function SuggestEditItemsSortAsc(List: TSuggestStrings; Index1, Index2: Integer): Integer;
begin
  Result := CompareStr(List[Index1], List[Index2]);
end;

function SuggestEditItemsSortDesc(List: TSuggestStrings; Index1, Index2: Integer): Integer;
begin
  Result := CompareStr(List[Index2], List[Index1]);
end;

function SuggestEditItemsSortLastUsed(List: TSuggestStrings; Index1, Index2: Integer): Integer;
begin
  Result := List.LastUseds[Index1] - List.LastUseds[Index2];
end;

{ TSuggestStringsItem }

class operator TSuggestStringsItem.Equal(Item1, Item2: TSuggestStringsItem): Boolean;
begin
  Result := Item1.Str = Item2.Str;
end;

procedure TSuggestStrings.SetSortingOrder(AValue: TSortingOrder);
begin
  if FSortingOrder = AValue then Exit;
  FSortingOrder := AValue;
  case AValue of
    soNone:     SortingFunction := Nil;
    soAsc:      SortingFunction := @SuggestEditItemsSortAsc;
    soDesc:     SortingFunction := @SuggestEditItemsSortDesc;
    soLastUsed: SortingFunction := @SuggestEditItemsSortLastUsed;
  end;
  Sort;
end;

function TSuggestStrings.GetLastUsed(Index: Integer): DWord;
begin
  Result := FList[Index].LastUsed;
end;

procedure TSuggestStrings.SetLastUsed(Index: Integer; AValue: DWord);
var
  aItem: TSuggestStringsItem;
begin
  aItem := FList[Index];
  aItem.LastUsed := AValue;
  FList[Index] := aItem;
end;

function TSuggestStrings.Get(Index: Integer): string;
begin
  Result := FList[Index].Str;
end;

function TSuggestStrings.GetCount: Integer;
begin
  Result := FList.Count;
end;

constructor TSuggestStrings.Create;
begin
  FList := TSuggestStringsItems.Create;
  FSortingFunction := Nil;
  FSortingOrder := soNone;
end;

destructor TSuggestStrings.Destroy;
begin
  FList.Free;
  inherited Destroy;
end;

procedure TSuggestStrings.Assign(Source: TPersistent);
var
  SL: TStrings;
  Used: array of Boolean;
  aIndex, i: Integer;
begin
  if Source is TStrings then
  begin
    SL := TStrings(Source);
    BeginUpdate;

    SetLength(Used, FList.Count);

    try
      for i := 0 to SL.Count - 1 do
      begin
        aIndex := IndexOf(SL[i]);
        if aIndex >= 0 then
        begin
          Used[aIndex] := True;
          Objects[aIndex] := SL.Objects[i];
          Continue;
        end;

        AddObject(SL[i], SL.Objects[i]);
        SetLength(Used, FList.Count);
        Used[FList.Count - 1] := True;
      end;

      for i := FList.Count - 1 downto 0 do
        if not Used[i] then
          Delete(i);

      Sort;
    finally
      EndUpdate;
    end;
  end
  else
    inherited Assign(Source);
end;

procedure TSuggestStrings.Clear;
begin
  FList.Clear;
end;

procedure TSuggestStrings.Delete(Index: Integer);
begin
  FList.Delete(Index);
end;

procedure TSuggestStrings.Insert(Index: Integer; const S: string);
var
  Item: TSuggestStringsItem;
begin
  Item.LastUsed := 0;
  Item.Obj := Nil;
  Item.Str := S;
  FList.Insert(Index, Item);
end;

procedure TSuggestStrings.SetLastUsedItem(ItemText: String);
var
  aIndex: Integer;
begin
  aIndex := IndexOf(ItemText);
  if aIndex >= 0 then
  begin
    LastUseds[aIndex] := DateTimeToLongWord(Now);
    Sort;
  end;
end;

procedure TSuggestStrings.Sort;
var
  i, j: Integer;
  Item: TSuggestStringsItem;
begin
  if SortingOrder = soNone then Exit;
  for i := 0 to Count - 2 do
    for j := i + 1 to Count - 1 do
      if SortingFunction(Self, i, j) < 0 then
      begin
        Item := FList[i];
        FList[i] := FList[j];
        FList[j] := Item;
      end;
end;


